package com.pastamadre.mypm;

import java.util.Iterator;
import java.util.Vector;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Path;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.PathShape;
import android.graphics.drawable.shapes.RectShape;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;

public class VistaJuego extends View {
	private Grafico misil;
	private static int PASO_VELOCIDAD_MISIL = 12;
	private boolean misilActivo = false;
	private int tiempoMisil;
	private Vector<Grafico> asteroides;
	private int numAsteroides = 6;
	private int numFragmentos = 4;
	private Grafico nave;
	private int giroNave;
	private double aceleracionNave;
	private static final int MAX_VELOCIDAD_NAVE = 10;
	private static final int PASO_GIRO_NAVE = 5;
	private static final float PASO_ACELERACION_NAVE = 0.5f;
	private ThreadJuego thread = new ThreadJuego();
	private static int PERIODO_PROCESO = 50;
	private long ultimoProceso = 0;
	SoundPool soundPool;
	SensorManager mSensorManager;
	Sensor orientationSensor;
	Sensor accelerometerSensor;
	int idDisparo, idExplosion, sonidosaltos;
	boolean sounds;
	int aleatorio;
	private int puntuacion = 0;
	private Drawable drawableAsteroide[] = new Drawable[3];

	public VistaJuego(Context context, AttributeSet attrs) {
		super(context, attrs);
		Drawable drawableNave, drawableMisil;
		soundPool = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);
		idDisparo = soundPool.load(context, R.raw.fiu, 0);
		idExplosion = soundPool.load(context, R.raw.sonidobotones2, 0);
		sonidosaltos = soundPool.load(context, R.raw.wee, 0);
		drawableAsteroide[0] = context.getResources().getDrawable(
				R.drawable.asteroide1);
		drawableAsteroide[1] = context.getResources().getDrawable(
				R.drawable.asteroide2);
		drawableAsteroide[2] = context.getResources().getDrawable(
				R.drawable.asteroide3);
		drawableNave = context.getResources().getDrawable(
				ElegirActividad2.imagenTamagotchi);
		aleatorio = (int) (Math.random() * 10 + 1);
		if (aleatorio < 4) {
			drawableMisil = context.getResources().getDrawable(
					R.drawable.misil1);
		} else if (aleatorio < 8) {
			drawableMisil = context.getResources().getDrawable(
					R.drawable.misil2);
		} else {
			drawableMisil = context.getResources().getDrawable(
					R.drawable.misil3);
		}

		nave = new Grafico(this, drawableNave);
		asteroides = new Vector<Grafico>();
		misil = new Grafico(this, drawableMisil);

		for (int i = 0; i < numAsteroides; i++) {

			Grafico asteroide = new Grafico(this, drawableAsteroide[0]);
			asteroide.setIncY(Math.random() * 6 - 3);
			asteroide.setIncX(Math.random() * 6 - 3);
			asteroide.setAngulo((int) (Math.random() * 360));
			asteroide.setRotacion((int) (Math.random() * 8 - 4));
			asteroides.add(asteroide);
		}
	}

	class ThreadJuego extends Thread {
		private boolean pausa, corriendo;

		public synchronized void pausar() {
			pausa = true;
		}

		public synchronized void reanudar() {
			pausa = false;
			notify();
		}

		public void detener() {
			corriendo = false;
			if (pausa)
				reanudar();
		}

		@Override
		public void run() {
			corriendo = true;
			while (corriendo) {
				actualizaFisica();
				synchronized (this) {
					while (pausa) {
						try {
							wait();
						} catch (Exception e) {
						}
					}
				}
			}
		}
	}

	@Override
	protected void onSizeChanged(int ancho, int alto, int ancho_anter,
			int alto_anter) {
		super.onSizeChanged(ancho, alto, ancho_anter, alto_anter);
		nave.setCenY(alto / 2);
		nave.setCenX(ancho / 2);

		for (Grafico asteroide : asteroides) {
			do {
				asteroide.setCenX((int) (Math.random() * ancho));
				asteroide.setCenY((int) (Math.random() * alto));
			} while (asteroide.distancia(nave) < (ancho + alto) / 5);
		}
		ultimoProceso = System.currentTimeMillis();
		thread.start();
	}

	@Override
	synchronized protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		for (Grafico asteroide : asteroides) {
			asteroide.dibujaGrafico(canvas);
		}
		if (misilActivo) {
			misil.dibujaGrafico(canvas);
		}
		nave.dibujaGrafico(canvas);
	}

	synchronized protected void actualizaFisica() {
		long ahora = System.currentTimeMillis();
		if (ultimoProceso + PERIODO_PROCESO > ahora) {
			return;
		}
		double retardo = (ahora - ultimoProceso) / PERIODO_PROCESO;
		ultimoProceso = ahora;
		nave.setAngulo((int) (nave.getAngulo() + giroNave * retardo));
		double nIncX = nave.getIncX() + aceleracionNave
				* Math.cos(Math.toRadians(nave.getAngulo())) * retardo;
		double nIncY = nave.getIncY() + aceleracionNave
				* Math.sin(Math.toRadians(nave.getAngulo())) * retardo;
		if (nIncX <= MAX_VELOCIDAD_NAVE) {
			nave.setIncX(nIncX);
			nave.setIncY(nIncY);
		}
		nave.incrementaPos(retardo);
		for (Grafico asteroide : asteroides) {
			asteroide.incrementaPos(retardo);
		}

		if (misilActivo) {
			misil.incrementaPos(retardo);
			tiempoMisil -= retardo;
			if (tiempoMisil < 0) {
				misilActivo = false;
			} else {
				for (int i = 0; i < asteroides.size(); i++)
					if (misil.verificaColision(asteroides.elementAt(i))) {
						destruyeAsteroide(i);
						break;
					}
			}
		}

		for (Grafico asteroide : asteroides) {
			if (asteroide.verificaColision(nave)) {
				soundPool.play(sonidosaltos, 1, 1, 1, 0, 2);
				salir();
			}
		}
	}

	private void destruyeAsteroide(int i) {
		int tam;
		if (asteroides.get(i).getDrawable() != drawableAsteroide[2]) {
			if (asteroides.get(i).getDrawable() == drawableAsteroide[1]) {
				tam = 2;
			} else {
				tam = 1;
			}
			for (int n = 0; n < numFragmentos; n++) {
				Grafico asteroide = new Grafico(this, drawableAsteroide[tam]);
				asteroide.setCenX(asteroides.get(i).getCenX());
				asteroide.setCenY(asteroides.get(i).getCenY());
				asteroide.setIncX(Math.random() * 7 - 2 - tam);
				asteroide.setIncY(Math.random() * 7 - 2 - tam);
				asteroide.setAngulo((int) (Math.random() * 360));
				asteroide.setRotacion((int) (Math.random() * 8 - 4));
				asteroides.add(asteroide);
			}
		}
		misilActivo = false;
		asteroides.remove(i);
		puntuacion += 10;
		soundPool.play(idExplosion, 1, 1, 0, 0, 1);

	}

	private void activaMisil() {
		soundPool.play(idDisparo, 1, 1, 1, 0, 1);
		misil.setCenX(nave.getCenX());
		misil.setCenY(nave.getCenY());
		misil.setAngulo(nave.getAngulo() + 90);
		misil.setIncX(Math.cos(Math.toRadians(misil.getAngulo()))
				* PASO_VELOCIDAD_MISIL);
		misil.setIncY(Math.sin(Math.toRadians(misil.getAngulo()))
				* PASO_VELOCIDAD_MISIL);

		tiempoMisil = (int) Math.min(
				this.getWidth() / Math.abs(misil.getIncX()), this.getHeight()
						/ Math.abs(misil.getIncY())) - 2;
		misilActivo = true;

	}

	private Activity padre;

	public void setPadre(Activity padre) {
		this.padre = padre;
	}

	private void salir() {
		Bundle bundle = new Bundle();
		bundle.putInt("puntuacion", puntuacion);
		Intent intent = new Intent();
		intent.putExtras(bundle);
		padre.setResult(Activity.RESULT_OK, intent);
		padre.finish();
	}

	private float mX = 0, mY = 0;
	private boolean disparo = false;

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		super.onTouchEvent(event);
		float x = event.getX();
		float y = event.getY();
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			disparo = true;
			break;
		case MotionEvent.ACTION_MOVE:
			float dx = Math.abs(x - mX);
			float dy = Math.abs(y - mY);
			if (dy < 6 && dx > 6) {
				giroNave = Math.round((x - mX));
				disparo = false;
			} else if (dx < 6 && dy > 6) {
				giroNave = Math.round((mY - y));
				disparo = false;
			}
			break;
		case MotionEvent.ACTION_UP:
			giroNave = 0;
			aceleracionNave = 0;
			if (disparo) {
				activaMisil();
			}
			break;
		}
		mX = x;
		mY = y;
		return true;
	}

	public ThreadJuego getThread() {
		return thread;
	}

}
